/*
 * spi_io.c
 *
 *  Created on: 2014-6-26
 *      Author: zhaoweibo
 */


/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif
#include <stdio.h>
#include <services/int/adi_sec.h>
#include <services/int/adi_int.h>
#include <drivers/spi/adi_spi.h>


#include <string.h>

/* Managed drivers and/or services include */
#include "system/adi_initialize.h"
#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic.")
#endif /* _MISRA_RULES */

static ADI_SPI_HANDLE phSPI;
uint8_t SpiMemory[ADI_SPI_DMA_MEMORY_SIZE];

uint32_t OpenDevice(uint32_t DeviceNum);
uint32_t CloseDevice(void);
uint32_t ConfigureDevice(void);
extern void ConfigSoftSwitches(void);

static bool s_bSPICompleted = false;

uint32_t OpenDevice(uint32_t DeviceNum)
{
	return (uint32_t)adi_spi_Open(DeviceNum, &SpiMemory, (uint32_t)ADI_SPI_DMA_MEMORY_SIZE, &phSPI);
}

uint32_t CloseDevice(void)
{
	return (uint32_t)adi_spi_Close(phSPI);
}

uint32_t ConfigureDevice(void)
{
	uint32_t Result = 0u;

    if (Result == 0u)
	{
       /* device in master of the SPI interface */
	   Result = (uint32_t)adi_spi_SetMaster(phSPI, true);
	}
    if (Result == 0u)
	{
       /* SPI slave select in controlled by software not hardware */
	   Result = (uint32_t)adi_spi_SetHwSlaveSelect(phSPI, false);
	}
    if (Result == 0u)
	{
    	/* send zeros if tx SPI underflows*/
		Result = (uint32_t)adi_spi_SetTransmitUnderflow(phSPI, true);
	}
    //add by zhwb
    if (Result == 0u)
	{
    	/* active high spi clk */
    	Result = (uint32_t)adi_spi_SetClockPolarity(phSPI, true);
	}
    if (Result == 0u)
	{
    	/* data transitions on falling edge of clock */
    	Result = (uint32_t)adi_spi_SetClockPhase(phSPI, false);
	}
    if (Result == 0u)
	{
    	/* SPI clock is SCLK divided by 1000 + 1 */
    	//Result = (uint32_t)adi_spi_SetClock(phSPI, 1000u);
    	Result = (uint32_t)adi_spi_SetClock(phSPI, 10u);
	}
    if (Result == 0u)
	{
    	/* SPI slave select is on SPI slave select 1 pin */
    	Result = (uint32_t)adi_spi_SetSlaveSelect(phSPI, ADI_SPI_SSEL_ENABLE1);	//by zhwb
	}
    if (Result == 0u)
	{
    	/* SPI data transfers are 8 bit */
    	Result = (uint32_t)adi_spi_SetWordSize(phSPI, ADI_SPI_TRANSFER_8BIT);
	}
    if (Result == 0u)
	{
    	/* generate tx data interrupt when watermark level breaches 50% level */
    	/* DMA watermark levels are disabled because SPI is in interrupt mode */
    	Result = (uint32_t)adi_spi_SetTxWatermark(phSPI,
                                                  ADI_SPI_WATERMARK_50,
                                                  ADI_SPI_WATERMARK_DISABLE,
                                                  ADI_SPI_WATERMARK_DISABLE);
	}
    if (Result == 0u)
	{
    	/* generate rx data interrupt when watermark level breaches 50% level */
    	/* DMA watermark levels are disabled because SPI is in interrupt mode */
    	Result = (uint32_t)adi_spi_SetRxWatermark(phSPI,
                                                  ADI_SPI_WATERMARK_50,
                                                  ADI_SPI_WATERMARK_DISABLE,
                                                  ADI_SPI_WATERMARK_DISABLE);
	}
	return Result;
}

void spi_Callback(void *pCBParam, uint32_t nEvent, void *pArg)
{
    switch(nEvent)
    {
        case (uint32_t)ADI_SPI_TRANSCEIVER_PROCESSED:

        	/* set callback flag */
            s_bSPICompleted = true;
            break;
        default:
            break;
    }
}

ADI_SPI_TRANSCEIVER trans;

int32_t zs_spi_read_write(uint8_t* pBuf, uint16_t nBufLen, uint8_t* pRpsBuf, uint16_t nRpsBufLen, uint8_t mode)
{
	if(NULL == pBuf || NULL == pRpsBuf)
		return -1;

	bool bAvailable = false;
	ADI_SPI_RESULT  ret;
	uint32_t error;
	ADI_SPI_TRANSCEIVER *pTrans;

	memset(&trans, 0x0, sizeof(trans));


	trans.pTransmitter = pBuf;
	trans.TransmitterBytes = nBufLen;
	trans.pReceiver = pRpsBuf;
	trans.ReceiverBytes = 0;

	s_bSPICompleted = false;
   	/* no callbacks */
    ret = adi_spi_RegisterCallback(phSPI, NULL, NULL);
	if(ADI_SPI_SUCCESS != ret)
	{
		printf("adi_spi_RegisterCallback failed!ret:%d\n", ret);
		return -1;
	}

   	/* interrupt mode, i.e no dma */
    ret = adi_spi_EnableDmaMode(phSPI, true);
	if(ADI_SPI_SUCCESS != ret)
	{
		printf("adi_spi_EnableDmaMode failed!ret:%d\n", ret);
		return -1;
	}


#if 0

	//adi_spi_SubmitBuffer

	if(ret == 0u)
	{
		ret = adi_spi_SubmitBuffer(phSPI, &trans);
	}

	printf("rx reg:%08X\n", *pREG_SPI1_RFIFO);

	int count = 0;
	while(1)//count < 50)
	{
		if(trans.ReceiverBytes > 0)
		{
			int i;
			printf("recv%d: ", trans.ReceiverBytes);
			for(i = 0; i < trans.ReceiverBytes; i ++)
			{
				printf("%02X ", trans.pReceiver[i]);

			}
			printf("\n");
			break;
		}

		rsi_delayMs(10);
		count ++;
	}
#if 1
	do{
			ret = adi_spi_IsBufferAvailable(phSPI, &bAvailable);
			rsi_delayMs(10);
	}while(!bAvailable);

	//rsi_delayMs(2000);
	/* get the pointer to the completed transceiver */
	if(ret == 0u)
	{
		ret = adi_spi_GetBuffer(phSPI, &pTrans);
	}
#endif
#endif

	ret = adi_spi_ReadWrite(phSPI, &trans);
	//printf("SPI1 Tx:%08X\n", *pREG_SPI1_TFIFO);
	if(ADI_SPI_SUCCESS != ret)
	{
		printf("adi spi readwrite failed!ret:%d\n", ret);
		return -1;
	}

	return 0;
}



int32_t zs_spi_read(uint8_t* pBuf, uint16_t nBufLen, uint8_t mode)
{
	ADI_SPI_RESULT res;
	ADI_SPI_TRANSCEIVER* pRecv;

	res = adi_spi_GetBuffer(phSPI, &pRecv);
	if(ADI_SPI_SUCCESS != res)
	{
		printf("adi spi get buffer failed! ret:%d", res);
		return -1;
	}

	if(pRecv->ReceiverBytes <= nBufLen)
		memcpy(pBuf, pRecv->pReceiver, pRecv->ReceiverBytes);
	else
		memcpy(pBuf, pRecv->pReceiver, nBufLen);

	return 0;
}


#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

